"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.createReducer = createReducer;
exports.default = configureStore;

var _reduxImmutable = require("redux-immutable");

var _redux = require("redux");

var _immutable = require("immutable");

var _reactRouterRedux = require("react-router-redux");

var _history = require("history");

var _reduxThunk = require("redux-thunk");

var _reduxThunk2 = _interopRequireDefault(_reduxThunk);

var _reduxSaga = require("redux-saga");

var _reduxSaga2 = _interopRequireDefault(_reduxSaga);

var _routeReducer = require("./utils/routeReducer");

var _routeReducer2 = _interopRequireDefault(_routeReducer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } } /**
                                                                                                                                                                                                     * Create the store with dynamic reducers
                                                                                                                                                                                                     */

var sagaMiddleware = (0, _reduxSaga2.default)();

/**
 * Creates the main reducer with the dynamically injected ones
 */
function createReducer(injectedReducers) {
  return (0, _reduxImmutable.combineReducers)(Object.assign({
    route: _routeReducer2.default
  }, injectedReducers));
}

function configureStore() {
  var reducers = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var initialState = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var history = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : (0, _history.createMemoryHistory)();
  var middlewares = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : [];

  // Create the store with 3 middlewares
  // 1. sagaMiddleware: Makes redux-sagas work
  // 2. thunkMiddleware: Makes redux-thunk work
  // 3. routerMiddleware: Syncs the location/URL path to the state
  var middlewaresToApply = [sagaMiddleware, _reduxThunk2.default, (0, _reactRouterRedux.routerMiddleware)(history)].concat(_toConsumableArray(middlewares));

  var enhancers = [_redux.applyMiddleware.apply(undefined, _toConsumableArray(middlewaresToApply))];

  // If Redux DevTools Extension is installed use it, otherwise use Redux compose
  /* eslint-disable no-underscore-dangle */
  var composeEnhancers = (typeof window === "undefined" ? "undefined" : _typeof(window)) === "object" && window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__ ? window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__({
    // TODO Try to remove when `react-router-redux` is out of beta, LOCATION_CHANGE should not be fired more than once after hot reloading
    // Prevent recomputing reducers for `replaceReducer`
    shouldHotReload: false
  }) : _redux.compose;
  /* eslint-enable */

  var store = (0, _redux.createStore)(createReducer(reducers), (0, _immutable.fromJS)(initialState), composeEnhancers.apply(undefined, enhancers));

  // Extensions
  store.runSaga = sagaMiddleware.run;
  store.injectedReducers = reducers; // Reducer registry
  store.injectedSagas = {}; // Saga registry
  store.history = history; // history

  // Make reducers hot reloadable, see http://mxs.is/googmo
  /* istanbul ignore next */
  if (module.hot) {
    module.hot.accept("./configureStore.js", function () {
      store.replaceReducer(createReducer(store.injectedReducers));
    });
  }

  return store;
}