"use strict";

var _enzyme = require("enzyme");

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _identity = require("lodash/identity");

var _identity2 = _interopRequireDefault(_identity);

var _configureDefaultStore = require("./configureDefaultStore");

var _configureDefaultStore2 = _interopRequireDefault(_configureDefaultStore);

var _injectReducer = require("./injectReducer");

var _injectReducer2 = _interopRequireDefault(_injectReducer);

var _reducerInjectors = require("./reducerInjectors");

var reducerInjectors = _interopRequireWildcard(_reducerInjectors);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Fixtures
/**
 * Test injectors
 */

var Component = function Component() {
  return null;
};

var reducer = _identity2.default;

describe("injectReducer decorator", function () {
  var store = void 0;
  var injectors = void 0;
  var ComponentWithReducer = void 0;

  beforeAll(function () {
    reducerInjectors.default = jest.fn().mockImplementation(function () {
      return injectors;
    });
  });

  beforeEach(function () {
    store = (0, _configureDefaultStore2.default)({}, {});
    injectors = {
      injectReducer: jest.fn()
    };
    ComponentWithReducer = (0, _injectReducer2.default)({ key: "test", reducer: reducer })(Component);
    reducerInjectors.default.mockClear();
  });

  it("should inject a given reducer", function () {
    (0, _enzyme.shallow)(_react2.default.createElement(ComponentWithReducer, null), { context: { store: store } });

    expect(injectors.injectReducer).toHaveBeenCalledTimes(1);
    expect(injectors.injectReducer).toHaveBeenCalledWith("test", reducer);
  });

  it("should set a correct display name", function () {
    expect(ComponentWithReducer.displayName).toBe("withReducer(Component)");
    expect((0, _injectReducer2.default)({ key: "test", reducer: reducer })(function () {
      return null;
    }).displayName).toBe("withReducer(Component)");
  });

  it("should propagate props", function () {
    var props = { testProp: "test" };
    var renderedComponent = (0, _enzyme.shallow)(_react2.default.createElement(ComponentWithReducer, props), {
      context: { store: store }
    });

    expect(renderedComponent.prop("testProp")).toBe("test");
  });
});