"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.injectReducerFactory = injectReducerFactory;
exports.default = getInjectors;

var _invariant = require("invariant");

var _invariant2 = _interopRequireDefault(_invariant);

var _isEmpty = require("lodash/isEmpty");

var _isEmpty2 = _interopRequireDefault(_isEmpty);

var _isFunction = require("lodash/isFunction");

var _isFunction2 = _interopRequireDefault(_isFunction);

var _isString = require("lodash/isString");

var _isString2 = _interopRequireDefault(_isString);

var _checkStore = require("./checkStore");

var _checkStore2 = _interopRequireDefault(_checkStore);

var _configureStore = require("../configureStore");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function injectReducerFactory(store, isValid) {
  return function injectReducer(key, reducer) {
    if (!isValid) {
      (0, _checkStore2.default)(store);
    }

    (0, _invariant2.default)((0, _isString2.default)(key) && !(0, _isEmpty2.default)(key) && (0, _isFunction2.default)(reducer), "(app/utils...) injectReducer: Expected `reducer` to be a reducer function");

    // Check `store.injectedReducers[key] === reducer` for hot reloading when a key is the same but a reducer is different
    if (Reflect.has(store.injectedReducers, key) && store.injectedReducers[key] === reducer) {
      return;
    }

    store.injectedReducers[key] = reducer; // eslint-disable-line no-param-reassign
    store.replaceReducer((0, _configureStore.createReducer)(store.injectedReducers));
  };
}

function getInjectors(store) {
  (0, _checkStore2.default)(store);

  return {
    injectReducer: injectReducerFactory(store, true)
  };
}