"use strict";

var _immutable = require("immutable");

var _identity = require("lodash/identity");

var _identity2 = _interopRequireDefault(_identity);

var _configureDefaultStore = require("./configureDefaultStore");

var _configureDefaultStore2 = _interopRequireDefault(_configureDefaultStore);

var _reducerInjectors = require("./reducerInjectors");

var _reducerInjectors2 = _interopRequireDefault(_reducerInjectors);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Fixtures

/**
 * Test injectors
 */

var initialState = (0, _immutable.fromJS)({ reduced: "soon" });

var reducer = function reducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
  var action = arguments[1];

  switch (action.type) {
    case "TEST":
      return state.set("reduced", action.payload);
    default:
      return state;
  }
};

describe("reducer injectors", function () {
  var store = void 0;
  var injectReducer = void 0;

  describe("getInjectors", function () {
    beforeEach(function () {
      store = (0, _configureDefaultStore2.default)({});
    });

    it("should return injectors", function () {
      expect((0, _reducerInjectors2.default)(store)).toEqual(expect.objectContaining({
        injectReducer: expect.any(Function)
      }));
    });

    it("should throw if passed invalid store shape", function () {
      Reflect.deleteProperty(store, "dispatch");

      expect(function () {
        return (0, _reducerInjectors2.default)(store);
      }).toThrow();
    });
  });

  describe("injectReducer helper", function () {
    beforeEach(function () {
      store = (0, _configureDefaultStore2.default)({});
      injectReducer = (0, _reducerInjectors.injectReducerFactory)(store, true);
    });

    it("should check a store if the second argument is falsy", function () {
      var inject = (0, _reducerInjectors.injectReducerFactory)({});

      expect(function () {
        return inject("test", reducer);
      }).toThrow();
    });

    it("it should not check a store if the second argument is true", function () {
      Reflect.deleteProperty(store, "dispatch");

      expect(function () {
        return injectReducer("test", reducer);
      }).not.toThrow();
    });

    it("should validate a reducer and reducer's key", function () {
      expect(function () {
        return injectReducer("", reducer);
      }).toThrow();
      expect(function () {
        return injectReducer(1, reducer);
      }).toThrow();
      expect(function () {
        return injectReducer(1, 1);
      }).toThrow();
    });

    it("given a store, it should provide a function to inject a reducer", function () {
      injectReducer("test", reducer);

      var actual = store.getState().get("test");
      var expected = initialState;

      expect(actual.toJS()).toEqual(expected.toJS());
    });

    it("should not assign reducer if already existing", function () {
      store.replaceReducer = jest.fn();
      injectReducer("test", reducer);
      injectReducer("test", reducer);

      expect(store.replaceReducer).toHaveBeenCalledTimes(1);
    });

    it("should assign reducer if different implementation for hot reloading", function () {
      store.replaceReducer = jest.fn();
      injectReducer("test", reducer);
      injectReducer("test", _identity2.default);

      expect(store.replaceReducer).toHaveBeenCalledTimes(2);
    });
  });
});