"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.injectSagaFactory = injectSagaFactory;
exports.ejectSagaFactory = ejectSagaFactory;
exports.default = getInjectors;

var _isEmpty = require("lodash/isEmpty");

var _isEmpty2 = _interopRequireDefault(_isEmpty);

var _isFunction = require("lodash/isFunction");

var _isFunction2 = _interopRequireDefault(_isFunction);

var _isString = require("lodash/isString");

var _isString2 = _interopRequireDefault(_isString);

var _invariant = require("invariant");

var _invariant2 = _interopRequireDefault(_invariant);

var _conformsTo = require("lodash/conformsTo");

var _conformsTo2 = _interopRequireDefault(_conformsTo);

var _checkStore = require("./checkStore");

var _checkStore2 = _interopRequireDefault(_checkStore);

var _constants = require("./constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var allowedModes = [_constants.RESTART_ON_REMOUNT, _constants.DAEMON, _constants.ONCE_TILL_UNMOUNT];

var checkKey = function checkKey(key) {
  return (0, _invariant2.default)((0, _isString2.default)(key) && !(0, _isEmpty2.default)(key), "(app/utils...) injectSaga: Expected `key` to be a non empty string");
};

var checkDescriptor = function checkDescriptor(descriptor) {
  var shape = {
    saga: _isFunction2.default,
    mode: function mode(_mode) {
      return (0, _isString2.default)(_mode) && allowedModes.includes(_mode);
    }
  };
  (0, _invariant2.default)((0, _conformsTo2.default)(descriptor, shape), "(app/utils...) injectSaga: Expected a valid saga descriptor");
};

function injectSagaFactory(store, isValid) {
  return function injectSaga(key) {
    var descriptor = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var args = arguments[2];

    if (!isValid) {
      (0, _checkStore2.default)(store);
    }

    var newDescriptor = Object.assign({}, descriptor, {
      mode: descriptor.mode || _constants.RESTART_ON_REMOUNT
    });
    var saga = newDescriptor.saga,
        mode = newDescriptor.mode;


    checkKey(key);
    checkDescriptor(newDescriptor);

    var hasSaga = Reflect.has(store.injectedSagas, key);

    if (process.env.NODE_ENV !== "production") {
      var oldDescriptor = store.injectedSagas[key];
      // enable hot reloading of daemon and once-till-unmount sagas
      if (hasSaga && oldDescriptor.saga !== saga) {
        oldDescriptor.task.cancel();
        hasSaga = false;
      }
    }

    if (!hasSaga || hasSaga && mode !== _constants.DAEMON && mode !== _constants.ONCE_TILL_UNMOUNT) {
      // eslint-disable-next-line no-param-reassign
      store.injectedSagas[key] = Object.assign({}, newDescriptor, {
        task: store.runSaga(saga, args)
      });
    }
  };
}

function ejectSagaFactory(store, isValid) {
  return function ejectSaga(key) {
    if (!isValid) {
      (0, _checkStore2.default)(store);
    }

    checkKey(key);

    if (Reflect.has(store.injectedSagas, key)) {
      var descriptor = store.injectedSagas[key];
      if (descriptor.mode !== _constants.DAEMON) {
        descriptor.task.cancel();
        // Clean up in production; in development we need `descriptor.saga` for hot reloading
        if (process.env.NODE_ENV === "production") {
          // Need some value to be able to detect `ONCE_TILL_UNMOUNT` sagas in `injectSaga`
          store.injectedSagas[key] = "done"; // eslint-disable-line no-param-reassign
        }
      }
    }
  };
}

function getInjectors(store) {
  (0, _checkStore2.default)(store);

  return {
    injectSaga: injectSagaFactory(store, true),
    ejectSaga: ejectSagaFactory(store, true)
  };
}